package lawOfDemeter;

import org.aspectj.lang.reflect.*;
import org.aspectj.lang.*;


/**
 * The GloballyPreferred aspect is part of a Law of Demeter Checker - a set of
 * classes and aspects that verify a given program does not violate
 * the Law of Demeter.  This aspect stores the classes returned to by
 * join points that are globally considered to be suppliers of prefered classes
 * for later examination by the Checker aspect.
 *
 * @author Paul Freeman
 */

public aspect GlobalPreferred extends LoDPointCutsContext {

  pointcut positiveSituationsContext(): Any.GlobalPreferred();

  Class[] getAllowed (JoinPoint.StaticPart ejsp,JoinPoint.StaticPart jsp) {
    Class thisType = null;
    Signature thisSig = jsp.getSignature();

    if(thisSig instanceof MethodSignature){
      thisType = ((MethodSignature)thisSig).getReturnType();
    }else if(thisSig instanceof FieldSignature){
      thisType = ((FieldSignature)thisSig).getFieldType();
    }else if(thisSig instanceof AdviceSignature) {
      thisType = ((AdviceSignature)thisSig).getReturnType();
    }

    if(thisType != null){
      Class[] v = new Class[1];
      v[0]= thisType;
      return v;
    }else{
      return null;
    }
  }
}