package edu.neu.ccs.com3205.lodchecker.internal;


import org.eclipse.ui.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.core.resources.*;
import org.eclipse.jdt.core.*;
import org.eclipse.jface.text.*;

import edu.neu.ccs.com3205.lodchecker.*;
import edu.neu.ccs.com3205.lodchecker.ui.*;

/**
 * LoDSelectionListener.java
 * This class keeps the currentProject information up to date
 * in the LoDLaunchListener class and the LoDCheckerViolationsView class
 * by monitoring any change the user makes to 
 * their selection in the workspace.<br>
 * 
 * @author Paul Freeman
 */

public class LoDSelectionListener implements ISelectionListener {

  private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();

	private static IProject currentProject = null;
	
	/**
	 * @return returns the project of the last selected item
	 */
	public static IProject getCurrentProject(){
		return currentProject;
	}
//	
// the following code was taken from the "org.eclipse.ajdt.ui" plugin
//
	// Implementation of ISelectionListener follows

	/** 
	 * Keeps the currentProject information up to date
	 * in the LoDLaunchListener class and the LoDCheckerViolationsView class
	 * as this method is called whenever a user changes
	 * their selection in the workspace.<br>
	 * <br>
	 * This method was taken and modified from code written for the AspectJ plugin.
	 */
	public void selectionChanged(IWorkbenchPart iwp, ISelection is) {
//		plugin.log("selection changed event received from "+iwp.getTitle());
//		plugin.log("selected: " + getSelectionString(is));
			
		try {
			// If we want to check only for selection changes in the Packages
			// view, then we could check the WorkbenchPart:
			//			if (iwp.getTitle().equals("Packages")) {
			// But there are so many places where the resources are exposed
			// navigator view, etc - that if we can be more generic and cope
			// with selection of the resources occurring anywhere, then we should act on
			// the current project correct.
				
			if (is instanceof IStructuredSelection) {
				IProject selectedProject = getProject((IStructuredSelection)is);
				
				if(selectedProject != null){
					currentProject = selectedProject;
					LoDLaunchListener.setCurrentProject(currentProject);
					LoDCheckerViolationsView lodView = LoDCheckerViolationsView.getActiveView();
					if(lodView != null){
						lodView.loadViolations(currentProject, false);
					}
				}
			}			
			
		} catch (Exception e) {
			plugin.log("Error handling selection", e);
		}
	}
	
	/**
	 * @return returns the project of the current selection if available or null 
	 * if it cannot be determined
	 */
	private IProject getProject(IStructuredSelection is){
		Object o = is.getFirstElement();
		if (o != null) {
			if (o instanceof IResource) {
				return ((IResource)o).getProject();

			} else if (o instanceof IJavaElement) {
				IJavaElement je = (IJavaElement) o;
				if (je.getJavaProject() != null) {
					return je.getJavaProject().getProject();
				}
			}
		}
		return null;	
	}
	
	/**
	 * Used in debugging.
	 * @return returns the selection as a String
	 */
	private String getSelectionString(ISelection is){
		Object obj;
		if(is instanceof IStructuredSelection){
			obj = ((IStructuredSelection)is).getFirstElement();
		}else if(is instanceof IMarkSelection){
			obj = ((IMarkSelection)is).getDocument();
		}else if(is instanceof ITextSelection){
			obj = ((ITextSelection)is).getText();
		}else{
			obj = is;
		}
		if(obj != null){
			return getFirstLine(obj.toString());
		}else{
			return null;
		}
	}
	
	/**
	 * Used in debugging
	 * @return returns the first line of mulitline string
	 */
	private String getFirstLine(String s){
		int newLinePos = s.indexOf("\n");
		if(newLinePos <= 0){ newLinePos = s.length(); }
		return s.substring(0, newLinePos);
	}
}
