package edu.neu.ccs.com3205.lodchecker.ui;

/**
 * WidgetMaker.java
 * This class makes the widgets common to both the Property page and the Violations view
 * 
 * @author Paul Freeman
 */

import org.eclipse.swt.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.events.*;

public class WidgetMaker {

//
// the following bits of code were taken from the ajdt plug-in and modified
	/**
	 * Helper method to build a labelled check box button.
	 * 
	 * @return returns the newly created checkbox button
	 */
	public Button createCheckBox(Composite container, String label, SelectionListener sl) {
		Button button = new Button(container, SWT.CHECK);
		button.setText(label);

		button.addSelectionListener(sl);
		GridData data = new GridData();
		data.horizontalSpan = 2;
		data.horizontalAlignment = GridData.FILL;
		button.setLayoutData(data);

		return button;
	}

	/**
	 * Helper method to build a labelled radio button.
	 * @return returns the newly created radio button.
	 */
	public Button createRadioButton(Composite parent, String label, SelectionListener sl) {
		Button button = new Button(parent, SWT.RADIO | SWT.LEFT);
		button.setText(label);

		button.addSelectionListener(sl);
		GridData data = new GridData();
		data.horizontalSpan = 2;
		button.setLayoutData(data);

		return button;
	}

	/**
	 * Helper method that creates a label instance.
	 *@return returns the newly created label
	 */
	public Label createLabel(Composite parent, String text) {
		Label label = new Label(parent, SWT.WRAP | SWT.LEFT); 
		label.setText(text);

		GridData data = new GridData();
		data.horizontalSpan = 2;
		data.horizontalAlignment = GridData.FILL;
		label.setLayoutData(data);

		return label;
	}

	/**
	* Creates composite control and sets the default layout data.
	*
	* @param parent  the parent of the new composite
	* @param numColumns  the number of columns for the new composite
	* @return the newly-created coposite
	*/
	public Composite createComposite(Composite parent, int numColumns) {
		Composite composite = new Composite(parent, SWT.NULL);

		//GridLayout
		GridLayout layout = new GridLayout();
		layout.numColumns = numColumns;
		composite.setLayout(layout);

		//GridData
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		composite.setLayoutData(data);

		return composite;
	}

}
