package edu.neu.ccs.com3205.lodchecker.utils;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.*;

/**
 * Messages.java
 * This class provides simple static message box implimentations.
 * 
 * @author Paul Freeman
 */
public class Messages {

	/**
	 * @return returns the shell of the current workbench window
	 */
	private static Shell getCurrentShell(){
    	IWorkbenchWindow window =
        	PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        if (window != null) {
        	return window.getShell();
        }else{
        	return null;
        }
	}
	
	/**
	 * opens a confirm messagebox with:
	 * the message argument
	 * @return returns true if the user presses ok and false otherwise
	 */
	public static boolean confirm(String message){
		Shell shell = getCurrentShell();
		if(shell != null){
			return MessageDialog.openConfirm(shell, "", message);
		}
		return false;
	}

	/**
	 * opens a confirm messagebox with:
	 * the title argument
	 * the message argument
	 * @return returns true if the user presses ok and false otherwise
	 */
	public static boolean confirm(String title, String message){
		Shell shell = getCurrentShell();
		if(shell != null){
			return MessageDialog.openConfirm(shell, title, message);
		}
		return false;
	}

	/**
	 * opens an error messagebox with:
	 * the message argument
	 * @return returns true if the user presses ok and false otherwise
	 */
	public static void error(String message){
		Shell shell = getCurrentShell();
		if(shell != null){
			MessageDialog.openError(shell, "", message);
		}
	}

	/**
	 * opens an error messagebox with:
	 * the title argument
	 * the message argument
	 * @return returns true if the user presses ok and false otherwise
	 */
	public static void error(String title, String message){
		Shell shell = getCurrentShell();
		if(shell != null){
			MessageDialog.openError(shell, title, message);
		}
	}

	/**
	 * opens an error messagebox with:
	 * the message argument
	 * the exception argument displayed in the message
	 * @return returns true if the user presses ok and false otherwise
	 */
	public static void error(String message, Exception e){
		Shell shell = getCurrentShell();
		if(shell != null){
			MessageDialog.openError(shell, "", message + "\nERROR: " + e);
		}
	}

	/**
	 * opens an error messagebox with:
	 * the title argument
	 * the message argument
	 * the exception argument displayed in the message
	 * @return returns true if the user presses ok and false otherwise
	 */
	public static void error(String title, String message, Exception e){
		Shell shell = getCurrentShell();
		if(shell != null){
			MessageDialog.openError(shell, title, message + "\nERROR: " + e);
		}
	}
	
}
