package edu.neu.ccs.com3205.lodchecker.utils;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;

import java.util.*;

import edu.neu.ccs.com3205.lodchecker.LoDCheckerPlugin;

/**
 * Natures.java
 * This class provides protected static methods for adding and removing 
 * Natures from resources.
 * 
 * @author Paul Freeman
 */
public class Natures {

  private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();

	/**
	 * adds a nature to a resource
	 * @return returns true if successfull or the nature already existed
	 * and false otherwise
	 */
	synchronized public static void addNature(IResource thisResource, String natureId) throws CoreException {
		try{
			// add nature if resource doesn't already have it
			if(!thisResource.getProject().hasNature(natureId)){
				//get natures
				IProject project = thisResource.getProject();
				IProjectDescription description = project.getDescription();
				String[] natures = description.getNatureIds();
				Vector newNatures = new Vector();
				newNatures.addAll(Arrays.asList(natures));
				
				// add nature to vector of natures
				newNatures.add(natureId);
	
				//update natures
				String[] natureArray = (String[])newNatures.toArray(new String[0]);
				description.setNatureIds(natureArray);
				project.setDescription(description, new NullProgressMonitor());
			}
		} catch (CoreException e) {
			// Something went wrong
			String message = "Unable to add nature: " + natureId;
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}		
	
	}
	
	/**
	 * removes a nature from a resource
	 * @return returns true if successfull or the nature did not exist 
	 * and false otherwise
	 */
	synchronized public static void removeNature(IResource thisResource, String natureId) throws CoreException {
		try {
     	// if project has nature remove it
			if(thisResource.getProject().hasNature(natureId)){
				// get natures
				IProject project = thisResource.getProject();
	     	IProjectDescription description = project.getDescription();
	     	String[] natures = description.getNatureIds();
	     	Vector newNatures = new Vector();
	     	newNatures.addAll(Arrays.asList(natures));
		     
		    // Remove nature from vector
			 	newNatures.remove(natureId);
				
				// update natures
	     	description.setNatureIds((String[])newNatures.toArray(new String[1]));
	     	project.setDescription(description, new NullProgressMonitor());
			}
		} catch (CoreException e) {
	    // Something went wrong
			String message = "Unable to remove nature: " + natureId;
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
	    }		
	}

}
